/**
 * ThumbNailedWrapper.java
 * Created on 2005/06/13
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.cache.impl;

import izayoi.Constants;
import izayoi.Util;
import izayoi.cache.CacheEntry;
import izayoi.cache.GenericRecord;
import izayoi.cache.Record;
import izayoi.cache.ThumbNailAppendableEntry;
import izayoi.cache.WritableRecord;
import izayoi.misc.ThumbNailMaker;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;

import sun.misc.BASE64Decoder;

/**
 * ̃Ggɏkډ摜@\t
 * @author es
 */
public class ThumbNailedWrapper extends ThumbNailAppendableEntry {

    private CacheEntry parent;
    private File root;
    private String format;

    /**
     * @param entry bvGg
     */
    public ThumbNailedWrapper(CacheEntry entry) {
        super(entry.getFactory(), false);
        parent = entry;
    }

    /**
     * @param entry bvGg
     * @param storedRoot kڂ̕ۑdir
     * @param type kڂ̉摜tH[}bg
     */
    public ThumbNailedWrapper(CacheEntry ent, File storedRoot, String type) {
        super(ent.getFactory(), true);
        parent = ent;
        root = storedRoot;
        format = type;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#makeThumbNail(izayoi.cache.Record)
     */
    protected void makeThumbNail(Record record) throws IOException {
        if (record == null
            || !record.containsField(Constants.FIELD_NAME_ATTACH)
            || !record.containsField(Constants.FIELD_NAME_SUFFIX)) {
            return;
        }
        String mime = Util.getMimeType("1." + record.getField(Constants.FIELD_NAME_SUFFIX));
        if (mime == null || !mime.startsWith("image/")) {
            return;
        }

        String filename = record.getTimeStamp() + "_" + record.getID() + "." + format;
        ByteArrayOutputStream bytes = new ByteArrayOutputStream();
        InputStream in = new StringInputStream(record.getField(Constants.FIELD_NAME_ATTACH));
        new BASE64Decoder().decodeBuffer(in, bytes);
        BufferedImage img = ImageIO.read(new ByteArrayInputStream(bytes.toByteArray()));
        if (img == null) {
            return;
        }

        new ThumbNailMaker(new File(root, filename), img, format).execute();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#append(izayoi.cache.Record)
     */
    public void appendRecord(Record record) throws IOException {
        if (!contains(record)) {
            parent.append(record);
        }
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#commit()
     */
    public void commit() throws IOException {
        parent.commit();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#contains(izayoi.cache.Record)
     */
    public boolean contains(Record record) throws IOException {
        return parent.contains(record);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getBaseName()
     */
    public String getBaseName() {
        return parent.getBaseName();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getCount()
     */
    public int getCount() throws IOException {
        return parent.getCount();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getFileName()
     */
    public String getFileName() {
        return parent.getFileName();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getLastModified()
     */
    public long getLastModified() throws IOException {
        return parent.getLastModified();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getRecordCount()
     */
    public int getRecordCount() throws IOException {
        return parent.getRecordCount();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getRecords()
     */
    public Record[] getRecords() throws IOException {
        return parent.getRecords();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getRecords(java.lang.String)
     */
    public Record[] getRecords(String argument) throws IOException {
        return parent.getRecords(argument);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getSize()
     */
    public long getSize() throws IOException {
        return parent.getSize();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getType()
     */
    public String getType() {
        return parent.getType();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#head(java.lang.String)
     */
    public String[] head(String argument) throws IOException {
        return parent.head(argument);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#isAutoCommit()
     */
    public boolean isAutoCommit() {
        return parent.isAutoCommit();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getRecord(java.lang.String)
     */
    public Record getRecord(String id) throws IOException {
        return parent.getRecord(id);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#setAutoCommit(boolean)
     */
    public void setAutoCommit(boolean b) {
        parent.setAutoCommit(b);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#sync(izayoi.cache.CacheEntry)
     */
    public void sync(CacheEntry entry) throws IOException {
        parent.sync(entry);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#deleteRecord(izayoi.cache.Record)
     */
    protected Record deleteRecord(Record record) throws IOException {
        parent.delete(record, false);
        WritableRecord r = new GenericRecord();
        r.setDeleteInfo(record.getTimeStamp(), record.getID());
        r.setTimeStamp(System.currentTimeMillis());
        return r;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#delete(izayoi.cache.Record, boolean)
     */
    public void delete(Record record, boolean b) throws IOException {
        if (!contains(record)) {
            return;
        }
        Record r = deleteRecord(record);
        if (b) {
            noticeMe(r);
        }
    }

}
class StringInputStream extends InputStream {
    int pos = 0;
    String src;
    StringInputStream(String src) {
        this.src = src;
    }
    public int read() throws IOException {
        if (pos >= src.length()) {
            return -1;
        }
        return src.charAt(pos++);
    }
}
