package izayoi.cache.impl.selene;

import izayoi.Util;
import izayoi.cache.CacheEntry;
import izayoi.cache.CacheFactory;
import izayoi.cache.GenericRecord;
import izayoi.cache.Record;
import izayoi.cache.WritableRecord;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;

import net.sourceforge.shingetsu.cache.Cache;
/**
 * replace of net.sourceforge.shingetsu.cache.Cache
 * @author es
 */
final class SeleneBindEntry extends CacheEntry {

    private String basename;
    private Cache entry;
    private String filename;
    private String type;

    SeleneBindEntry(CacheFactory fc, File file) throws IOException {
        super(fc);
        try {
            String str[] = file.getName().split("_");
            this.type = str[0];
            if(str[1].indexOf(".") != -1) {
                str[1] = str[1].substring(0, str[1].indexOf("."));
            }
            this.basename = Util.parseString(str[1]);
            filename = file.getName();
            entry = new Cache(file);
        } catch (NullPointerException e) {
            throw new IOException(e.getMessage());
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new IOException("wrong filename");
        }
    }

    SeleneBindEntry(CacheFactory fc, File file, String type, String basename) throws IOException {
        super(fc);
        try {
            this.type = type;
            this.basename = basename;
            filename = type + "_" + Util.parseHexString(basename);
            entry = new Cache(file);
        } catch (NullPointerException e) {
            throw new IOException(e.getMessage());
        }
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#append(izayoi.cache.CacheRecord)
     */
    public void append(Record record) throws IOException {
        synchronized (entry) {
            if (!contains(record) && record != null) {
                entry.append(record.parseString());
            }
        }
    }

    /**
     * ̎ł͌ĂяoKv͂Ȃ
     * @see izayoi.cache.CacheEntry#commit()
     */
    public void commit() throws IOException {
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#contains(izayoi.cache.CacheRecord)
     */
    public boolean contains(Record record) throws IOException {
    	if(record == null) {
    		return false;
    	}
        String id = record.isDeletedRecord() ? record.getDeleteRequestID() : record.getID();
        long time = record.isDeletedRecord() ? record.getDeletedTime() : record.getTimeStamp();
        return entry.containsRecord(time, id);
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#delete(izayoi.cache.CacheRecord)
     */
    protected Record deleteRecord(Record record) throws IOException {
        entry.deleteRecord(record.getTimeStamp(), record.getID());
        WritableRecord r = new GenericRecord();
        r.setDeleteInfo(record.getTimeStamp(), record.getID());
        r.setTimeStamp(System.currentTimeMillis());
        return r;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getBaseName()
     */
    public String getBaseName() {
        return basename;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getCount()
     */
    public int getCount() throws IOException {
        return entry.getRecordCount();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getFileName()
     */
    public String getFileName() {
        return filename;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getLastModified()
     */
    public long getLastModified() throws IOException {
        Record r = getLastRecord();
        if (r != null) {
            return r.getTimeStamp();
        } else {
            return entry.getFile().lastModified() / 1000;
        }
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getRecord(java.lang.String)
     */
    public Record getRecord(String id) throws IOException {
        Record[] r = getRecords();
        Comparator c = new IDcomparator();
        Arrays.sort(r, c);
        int i = Arrays.binarySearch(r, id, c);
        if (i >= 0) {
            return r[i];
        } else {
            return null;
        }
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getRecordCount()
     */
    public int getRecordCount() throws IOException {
        return entry.getRecordCount();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getRecords()
     */
    public Record[] getRecords() throws IOException {
        Record[] r = GenericRecord.parseRecord(entry.getRecords());
        return r;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#search(java.lang.String)
     */
    public Record[] getRecords(String argument) throws IOException {
        Record[] r = GenericRecord.parseRecord(entry.searchRecord(argument));
        return r;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getSize()
     */
    public long getSize() throws IOException {
        return entry.getFile().length();
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#getType()
     */
    public String getType() {
        return type;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#head(java.lang.String)
     */
    public String[] head(String argument) throws IOException {
        Record[] record = getRecords(argument);
        String[] ret = new String[record.length];
        for (int i = 0; i < record.length; i++) {
            ret[i] = record[i].getHeader();
        }
        return ret;
    }

    /**
     * ̎ł͏true
     * @see izayoi.cache.CacheEntry#isAutoCommit()
     */
    public boolean isAutoCommit() {
        return true;
    }

    /**
     * ̎ł͖
     * @see izayoi.cache.CacheEntry#setAutoCommit(boolean)
     */
    public void setAutoCommit(boolean b) {
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheEntry#sync(izayoi.cache.CacheEntry)
     */
    public void sync(CacheEntry entry) throws IOException {
        // TODO ꂽ\bhEX^u
    }

    class IDcomparator implements Comparator {

        /* ( Javadoc)
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        public int compare(Object arg0, Object arg1) {
            String a0, a1;
            if (arg0 instanceof Record) {
                a0 = ((Record) arg0).getID();
            } else {
                a0 = arg0.toString();
            }
            if (arg1 instanceof Record) {
                a1 = ((Record) arg1).getID();
            } else {
                a1 = arg1.toString();
            }
            return a0.compareTo(a1);
        }
    }

}
