/**
 * Client.java
 * Created on 2005/06/17
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.client;

import izayoi.Constants;
import izayoi.cache.CacheEntry;
import izayoi.cache.CacheFactory;
import izayoi.cache.Record;
import izayoi.node.Node;
import izayoi.node.NodeController;
import izayoi.node.NodeControllerFactory;

import java.io.IOException;
import java.lang.reflect.Method;
import java.net.ProtocolException;
import java.net.URL;

import edu.emory.mathcs.backport.java.util.Queue;
import edu.emory.mathcs.backport.java.util.concurrent.LinkedBlockingQueue;

/**
 * @author es
 */
public class Client {

    private class Executer extends Thread {
        final Client client;

        Executer(Client c) {
            client = c;
        }

        /* ( Javadoc)
         * @see java.lang.Runnable#run()
         */
        public void run() {
            while (true) {
                MethodHolder m = (MethodHolder) _cue.poll();
                if (m != null) {
                    try {
                        m.method.invoke(client, m.arg);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                try {
                    if (this.isInterrupted()) {
                        break;
                    }
                    sleep(5000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                    break;
                }
            }
        }
    }

    private class MethodHolder {
        Object[] arg;
        Method method;

        MethodHolder(Method method, Object[] arg) {
            this.method = method;
            this.arg = arg;
        }
    }

    private static final Client _me = new Client();
    private Queue _cue = new LinkedBlockingQueue();
    private Thread _thread;
    private NodeController _node;
    private CacheFactory _cache;

    public static Client getInstance() {
        return _me;
    }

    private Client() {
        _node = NodeControllerFactory.getInstance();
        _cache = CacheFactory.getInstance();
        _thread = new Executer(this);
        _thread.start();
    }

    /**
     * @param method
     * @param arg
     */
    public void addQueue(String method, String[] arg) {
        if (arg == null) {
            return;
        }
        Class[] c = new Class[] { arg.getClass()};
        try {
            Method m = this.getClass().getMethod(method, c);
            _cue.add(new MethodHolder(m, new Object[] { arg }));
        } catch (Exception e) {
            IllegalArgumentException ex = new IllegalArgumentException();
            ex.initCause(e);
            throw ex;
        }
    }

    /**
     * @return
     */
    public boolean hasQueue() {
        return _cue.size() != 0;
    }

    /**
     * 
     */
    public synchronized void stop() {
        _thread.interrupt();
        _thread = null;
    }

    public void symphasize(String[] arg) throws IOException {
        if (arg == null || arg.length < 2) {
            return;
        }

        String type = arg[0];
        String basename = arg[1];
        String nodeUrl = (arg.length > 2) ? arg[2] : null; 
        Node node = (nodeUrl != null) ? _node.getNode(new URL(nodeUrl)) : _node.getRandomNode();
        if (node == null) {
            return;
        }

        CacheEntry cache = _cache.getEntry(type, basename, true);
        long l;
        if (cache.getSize() > 0) {
            l = cache.getLastModified();
        } else {
            l = 0;
        }

        Record[] r = null;
        try {
             r = node.get(cache.getFileName(), l + "-");
        } catch (ProtocolException e) {
            e.printStackTrace();
        }
        if(r == null) {
            return;
        }

        if (type.equals(Constants.TYPE_LIST)) {
            for (int i = 0; i < r.length; i++) {
                if (!r[i].isDeletedRecord()) {
                    String lname = r[i].getField(Constants.FIELD_NAME_LINK);
                    String ltype = r[i].getField(Constants.FIELD_NAME_TYPE);
                    String[] param;
                    if(nodeUrl != null) {
                        param = new String[] { lname, ltype, nodeUrl};                        
                    } else {
                        param = new String[] { lname, ltype };
                    }
                    addQueue("symphasize", param);
                }
            }
        }

        cache.append(r);
    }
}
